SystemOrganization addCategory: #'AL-Core-Kernel'!
SystemOrganization addCategory: #'AL-Core-Exceptions'!

Exception subclass: #ALException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Core-Exceptions'!

ALException subclass: #ALError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Core-Exceptions'!

!ALError methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2008 23:00'!
defaultAction
	UnhandledError signalForException: self! !

!ALError methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2008 23:00'!
isResumable 
	^ false! !

ALError subclass: #ALNotFound
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Core-Exceptions'!

!ALNotFound methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2008 23:01'!
handleWith: anObject
	^ anObject handleNotFound: self! !

Object subclass: #ALApplication
	instanceVariableNames: 'locator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Core-Kernel'!

!ALApplication class methodsFor: 'as yet unclassified' stamp: 'cwp 3/7/2011 23:08'!
withLocator: aLocator
	^ self basicNew
		initializeWithLocator: aLocator;
		yourself! !

!ALApplication methodsFor: 'error handling' stamp: 'cwp 4/23/2011 18:44'!
handleNotFound: anError
	^ ALResponse notFound: anError messageText! !

!ALApplication methodsFor: 'api' stamp: 'cwp 4/23/2011 17:29'!
handleRequest: aRequest
	| path |
	path := aRequest path.
	^ [(locator resourceForPath: path) handleRequest: aRequest locator: locator]
		on: ALError
		do: [:err | err handleWith: self]
! !

!ALApplication methodsFor: 'initialize-release' stamp: 'cwp 3/7/2011 23:08'!
initializeWithLocator: aLocator
	locator := aLocator! !

!ALApplication methodsFor: 'responses' stamp: 'cwp 3/16/2011 11:18'!
responseFromFile: aReference
	| buffer entity |
	entity := ZnLazyEntity do:
		[:out |
		aReference fileStreamWritable: false do: 
			[:in | 
			buffer := in contentsOfEntireFile.
			out nextPutAll: buffer]].
	entity contentLength: aReference entry size.
	entity contentType: 'text/plain'.
	^ ZnResponse ok: entity! !

Object subclass: #ALLocator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Core-Kernel'!

ALLocator subclass: #ALClientLocator
	instanceVariableNames: 'index external'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Core-Kernel'!

!ALClientLocator methodsFor: 'initialize-release' stamp: 'cwp 3/11/2011 11:30'!
initializeWithIndex: aResource
	self initialize.
	index := aResource.
	external := IdentityDictionary new.
! !

!ALClientLocator methodsFor: 'public' stamp: 'cwp 3/11/2011 14:55'!
pathForResource: aResource 
	| archive |
	archive := SMSerializer serialize: aResource external: external.
	^ ALPath with: archive base64! !

!ALClientLocator methodsFor: 'public' stamp: 'cwp 3/11/2011 15:16'!
resourceForPath: aPath
	| bytes archive array |
	aPath isRoot ifTrue: [^ index].
	bytes := (Base64MimeConverter mimeDecodeToBytes: aPath first readStream) contents.
	array := Array new: external size.
	external keysAndValuesDo: [:object :i | array at: i put: object].
	archive := SMArchive bytes: bytes external: array.
	^ archive materialize! !

ALLocator subclass: #ALExplicitLocator
	instanceVariableNames: 'map index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Core-Kernel'!

!ALExplicitLocator methodsFor: 'accessing' stamp: 'cwp 3/7/2011 23:35'!
at: name put: aResource
	map at: name put: aResource ! !

!ALExplicitLocator methodsFor: 'initialize-release' stamp: 'cwp 3/11/2011 11:32'!
initializeWithIndex: aResource
	self initialize.
	index := aResource.
	map := Dictionary new.
	
! !

!ALExplicitLocator methodsFor: 'public' stamp: 'cwp 3/11/2011 15:09'!
pathForResource: aResource
	| name |
	name := map keyAtValue: aResource ifAbsent: [ALNotFound signal].
	^ ALPath with: name! !

!ALExplicitLocator methodsFor: 'public' stamp: 'cwp 3/11/2011 15:04'!
resourceForPath: aPath
	^ aPath isRoot 
		ifTrue: [index]
		ifFalse: [map at: (aPath at: 1) ifAbsent: [ALNotFound signal]]! !

ALLocator subclass: #ALKeyLocator
	instanceVariableNames: 'external resources entryPoint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Core-Kernel'!

!ALKeyLocator methodsFor: 'initialize-release' stamp: 'cwp 3/11/2011 11:27'!
initializeWithIndex: resource
	| archive |
	self initialize.
	resources := Dictionary new.
	external := IdentityDictionary new.
	archive := SMSerializer serialize: resource external: external.
	entryPoint := archive digest.
	resources at: entryPoint put: archive! !

!ALKeyLocator methodsFor: 'public' stamp: 'cwp 3/11/2011 14:48'!
pathForResource: aResource
	| archive digest |
	archive := SMSerializer serialize: aResource external: external.
	digest := archive digest.
	resources at: digest put: archive.
	^ ALPath with: digest! !

!ALKeyLocator methodsFor: 'public' stamp: 'cwp 3/11/2011 17:22'!
resourceForPath: path
	| key |
	key := path size = 1 ifTrue: [path at: 1] ifFalse: [entryPoint].
	^ (resources 
		at: key
		ifAbsent: [ ALNotFound signal: path ]) materialize! !

!ALLocator class methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 11:26'!
withIndex: aResource
	^ self basicNew initializeWithIndex: aResource ! !

!ALLocator methodsFor: 'initialize-release' stamp: 'cwp 3/11/2011 11:23'!
initializeWithIndex: aResource
	self subclassResponsibility ! !

!ALLocator methodsFor: 'public' stamp: 'cwp 9/25/2008 08:09'!
pathForResource: aResource
	self subclassResponsibility! !

!ALLocator methodsFor: 'public' stamp: 'cwp 9/25/2008 08:09'!
resourceForPath: anArray
	self subclassResponsibility ! !

Object variableSubclass: #ALPath
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Core-Kernel'!

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 3/20/2011 22:57'!
/ aString
	^ self with: aString! !

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 14:31'!
root
	^ self new: 0! !

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 14:48'!
with: aString
	^ (self new: 1)
		at: 1 put: aString;
		yourself! !

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 17:15'!
withAll: aCollection
	| path |
	path := self new: aCollection size.
	aCollection withIndexDo: [:element :index | path at: index put: element].
	^ path! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 7/21/2011 11:42'!
/ aString
	| path |
	aString isEmpty ifTrue: [^ self].
	path := self class new: self size + 1.
	1 to: self size do: [:i | path at: i put: (self at: i)].
	path at: (path size) put: aString.
	^ path! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 3/21/2011 22:20'!
= other
	self species = other species ifFalse: [^ false].
	self size = other size ifFalse: [^ false].
	1 to: self size do: [:i | (self at: i) = (other at: i) ifFalse: [^ false]].
	^ true! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 15:17'!
first
	^ self at: 1! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 3/21/2011 22:22'!
hash
	| hash |
	hash := self class hash.
	1 to: self size do: [:i | hash := hash bitXor: (self at: i) hash].
	^ hash! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 14:54'!
isRoot
	^ self size = 0! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 15:21'!
printOn: aStream
	1 to: self size do:
		[:index |
		aStream 
			nextPut: $/;
			nextPutAll: (self at: index)]! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 15:18'!
rest
	| rest |
	rest := self class new: self size - 1.
	1 to: rest size do: [:index | rest at: index put: (self at: index + 1)].
	^ rest! !

Object subclass: #ALResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Core-Kernel'!

ALResource subclass: #ALCallbackResource
	instanceVariableNames: 'resource callback'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Core-Kernel'!

!ALCallbackResource class methodsFor: 'as yet unclassified' stamp: 'cwp 8/5/2011 22:19'!
resource: aResource callback: aValuable
	^ self basicNew initializeWithResource: aResource callback: aValuable! !

!ALCallbackResource methodsFor: 'as yet unclassified' stamp: 'cwp 8/5/2011 22:20'!
initializeWithResource: aResource callback: aValuable
	self initialize.
	resource := aResource.
	callback := aValuable.! !

ALResource subclass: #ALDataResource
	instanceVariableNames: 'facet'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Core-Kernel'!

!ALDataResource class methodsFor: 'as yet unclassified' stamp: 'cwp 7/21/2011 11:36'!
facet: aFacet
	^ self basicNew initializeWithFacet: aFacet! !

!ALDataResource methodsFor: 'public' stamp: 'cwp 8/5/2011 22:10'!
handleRequest: aRequest locator: aLocator
	
	^ ALResponse 
		type: 'application/json'
		callback: 
			[:out || utf8 context |
			utf8 := out encoding: #utf8.
			context := (ALRenderingContext resource: self locator: aLocator).
			facet renderOn: (ALJsonRenderer context: context stream: utf8)]
	! !

!ALDataResource methodsFor: 'initialize-release' stamp: 'cwp 7/21/2011 11:36'!
initializeWithFacet: aFacet
	facet := aFacet! !

ALResource subclass: #ALFileResource
	instanceVariableNames: 'reference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Core-Kernel'!

!ALFileResource class methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 23:17'!
on: aReference
	^ self new initializeWithReference: aReference! !

!ALFileResource methodsFor: 'comparing' stamp: 'cwp 3/11/2011 23:18'!
= other 
	^ self species = other species and: [self reference = other reference]! !

!ALFileResource methodsFor: 'public' stamp: 'cwp 3/11/2011 23:20'!
handleRequest: aRequestContext for: anAdaptor
	^ anAdaptor responseFromFile: reference! !

!ALFileResource methodsFor: 'comparing' stamp: 'cwp 3/11/2011 23:18'!
hash
	^ reference hash! !

!ALFileResource methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 23:18'!
initializeWithReference: aReference
	reference := aReference ! !

!ALFileResource methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 23:18'!
reference
	^ reference! !

ALResource subclass: #ALFormResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Core-Kernel'!

!ALFormResource class methodsFor: 'as yet unclassified' stamp: 'cwp 9/12/2008 23:41'!
with: aPageResource do: defaultAction
	^ self new setPage: aPageResource defaultAction: defaultAction! !

!ALFormResource methodsFor: 'public' stamp: 'cwp 9/14/2008 22:30'!
applyRequest: aContext
	callbacks executeWithFields: aContext fields.
	^ page! !

!ALFormResource methodsFor: 'public' stamp: 'cwp 9/12/2008 23:45'!
callbacks
	^ callbacks! !

!ALFormResource methodsFor: 'initialize-release' stamp: 'cwp 9/12/2008 23:45'!
setPage: aPageResource defaultAction: aBlock
	page := aPageResource.
	defaultAction := aBlock.
	callbacks := ALCallbackRegistry new.! !

ALResource subclass: #ALLinkResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Core-Kernel'!

!ALLinkResource class methodsFor: 'as yet unclassified' stamp: 'cwp 9/9/2008 00:42'!
with: aPageResource do: aBlock
	^ self new setPage: aPageResource callback: aBlock! !

!ALLinkResource methodsFor: 'public' stamp: 'cwp 9/20/2008 20:52'!
applyRequest: aRequest
	^ self value.
	! !

!ALLinkResource methodsFor: 'public' stamp: 'cwp 9/23/2008 22:29'!
component
	^ page component! !

!ALLinkResource methodsFor: 'initialize-release' stamp: 'cwp 9/21/2008 00:55'!
setPage: aPageResource callback: aBlock
	page := aPageResource.
	callback := aBlock fixCallbackTemps ! !

!ALLinkResource methodsFor: 'public' stamp: 'cwp 9/21/2008 19:36'!
value
	ALSubcomputation do:
		[:sc |
		[callback copy value]
			on: ALSubcomputationRequest 
			do: [:request | request resume: sc]].
	
	^ page ! !

ALResource subclass: #ALPageResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Core-Kernel'!

!ALPageResource class methodsFor: 'as yet unclassified' stamp: 'cwp 9/8/2008 22:05'!
on: aComponent
	^ self new setComponent: aComponent! !

!ALPageResource methodsFor: 'public' stamp: 'cwp 9/13/2008 20:17'!
applyRequest: aContext
	^ self! !

!ALPageResource methodsFor: 'accessing' stamp: 'cwp 9/12/2008 00:04'!
component
	^ component! !

!ALPageResource methodsFor: 'accessing' stamp: 'cwp 9/9/2008 00:03'!
document
	^ ALHtmlStreamDocument new
		stream: String new writeStream ;
		yourself! !

!ALPageResource methodsFor: 'public' stamp: 'cwp 3/11/2011 11:16'!
handleRequest: aRequestContext for: anAdaptor
	| canvas context |
	context := aRequestContext renderingContextForPage: self.
	canvas := ALApplicationCanvas context: context.
	canvas document: self document.
	self renderOn: canvas.
	^ anAdaptor responseOk: canvas contents! !

!ALPageResource methodsFor: 'public' stamp: 'cwp 10/14/2008 00:39'!
renderOn: html
	(html tag: 'html') with:
		[html 
			head: 
				[html title: (component titleInContext: html context).
				component renderEffectiveHeadOn: html];
			body: 
				[component renderEffectiveBodyOn: html]]! !

!ALPageResource methodsFor: 'initialize-release' stamp: 'cwp 9/8/2008 22:05'!
setComponent: aComponent
	component := aComponent! !

!ALResource methodsFor: 'public' stamp: 'cwp 4/23/2011 17:30'!
handleRequest: aRequest locator: aLocator
	| path |
	path := aLocator pathForResource: (self applyRequest: aRequest).
	^ ALRequest seeOther: path! !
